scriptName LZStringUtil Hidden

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; AUTHOR: Lord Zapharos                                                                         ;;
;; VERSION: 1.0                                                                                  ;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; This SKSE plugin defines string operations that did not exist in the original SKSE StringUtil ;;
;; script, including the ability to split a string by delimiters and convert a reference's form  ;;
;; ID into a hex-string representation usable by TES5Edit. This plugin also provides support for ;;
;; form strings that are independent of load order.                                              ;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;





;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Splits the given string into an array, delimited by a character delimiter.
;; If no delimiter is found, this function returns an array with one element (the original string).
;;
;; The delimiter must be a single character. For example, if toSplit == "ab;c;;def;g;" and the
;; delimiter == ";", an array with the following elements is produced:
;;
;; string[
;;  "ab",
;;  "c",
;;  "def",
;;  "g"
;; ]
;;
;; If the provided string ONLY contains delimiters, this function will return a zero-length array.
;; Delimiters are CASE-SENSITIVE (i.e. "a" and "A" are not considered the same).
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
string[] Function Split(string toSplit, string delimiter) global native


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Obtains a four-letter string corresponding to the record type codes displayed within TES5Edit
;;  (e.g. "WEAP" for weapons), but only if the object is *placeable*.
;; Non-placeable objects will return an empty (zero-length) string.
;;
;; Placeable objects include weapons, soul gems, armor, furniture, statics, and lights.
;; Non-placeable objects instead include actors, sound references, global values, and so on
;;  (i.e. objects the main player cannot pick up, interact with, or collide with).
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
string Function GetPlaceableTypeTag(int objType) global native


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Determines whether the given object's hexadecimal form ID corresponds to a custom-placed object.
;; A custom object may have been placed via the player (e.g. dropped items) or by a script, but in
;;  all cases DOES NOT exist as a predefined reference in any ESP or ESM.
;; If such an object is selected in the console, its form ID will always begin with "FF".
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
bool Function IsCustomObjectHex(string formIDHex) global native


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Determines whether given object's form ID corresponds to a custom-placed object.
;; This function is otherwise identical to IsCustomObjectHex(), but first converts the provided
;;  formIDDec value into its hexadecimal representation
;;  (the same representation as seen when selecting objects in the console).
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
bool Function IsCustomObjectDec(int formIDDec) global native


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Given an object's hexadecimal form ID, extracts the index corresponding to its load order.
;; The resulting value can be used for functions like Game.GetModName().
;; Custom-placed objects will always return a value of 255.
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
int Function GetModIndex(string formIDHex) global native


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Converts a load-order-independent form ID string into its integer equivalent.
;; This function expects a string value that was previously produced by GetLoadIndependentFormHex()
;;  or GetLoadIndependentFormDec().
;; The returned value will be suitable for use with functions like Game.GetFormEx().
;;
;; This function is completely agnostic to load order.
;; If a form ID refers to "MyModName.esp", the correct form ID will always be obtained regardless
;;  of whether "MyModName.esp" is loaded at position 0A or position FE.
;; This function will only fail if "MyModName.esp" does not exist in the load order at all,
;;  in which case the form ID 0 i.e. 0x00000000 is returned.
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
int Function ResolveLoadIndependentForm(string indFormID) global native


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Converts an object's hexadecimal form ID into a load-order-independent string, suitable for
;;  writing to a file or for passing values between different mods.
;;
;; Custom-placed objects (i.e. those with form IDs prefixed by FF)
;;  must NEVER be passed into this function.
;; If you provide a custom-placed object, this function will return an empty string.
;;
;; This function and ResolveLoadIndependentForm() allow you to completely ignore the order
;;  in which mods have been loaded, which is very useful when saving form IDs to a file or when
;;  sharing data between different mods.
;;
;; For example, suppose this function is called on form ID 0x0A059381, originating from the mod
;;  "MyModName.esp" that is currently loaded as position 0A in the load order.
;; Suppose the resulting value is saved to a file (or within the save file), and then the user
;;  installs several new mods, changing "MyModName.esp" from position 0A to position FE.
;;
;; If you had saved form ID 0x0A059381 directly, your script would no longer work because 0A does
;;  not point to "MyModName.esp" anymore!
;; But by calling this function first, saving the resulting value, and later calling
;;  ResolveLoadIndependentForm(), the saved value will correctly point to the new location of
;;  "MyModName.esp" (now at FE), and your script can proceed as if the load order had not changed
;;  at all.
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
string Function GetLoadIndependentFormHex(string formIDHex) global native


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Does exactly the same thing as GetLoadIndependentFormHex(), but accepts an integer form ID
;;  instead (e.g. as obtained by MyObj.GetFormID()).
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
string Function GetLoadIndependentFormDec(int formIDDec) global native


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Identical to ResolveLoadIndependentForm(), but permits custom-placed objects.
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
int Function ResolveLoadIndependentFormAllowCustom(string indFormID) global native


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Identical to GetLoadIndependentForm(), but permits custom-placed objects.
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
string Function GetLoadIndependentFormHexAllowCustom(string formIDHex) global native


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Identical to GetLoadIndependentFormDec(), but permits custom-placed objects.
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
string Function GetLoadIndependentFormDecAllowCustom(int formIDDec) global native


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Converts an object's integer form ID (e.g. as obtained by MyObj.GetFormID()) into a hexadecimal
;;  representation, as would be shown when selecting the object in the console.
;; The resulting value can be used to directly find non-custom objects in TES5Edit
;;
;; If you intend to write these values to a file, it is better to call GetLoadIndependentFormDec()
;;  instead, as the latter function will create a representation that is agnostic to load order.
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
string Function FormToHex(int formID) global native


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Converts an integer into its hexadecimal representation, with a fixed width in bytes.
;; Example behaviors:
;;
;;  DecToHex(10, 1) produces "0A"
;;  DecToHex(10, 4) produces "0000000A"
;;  DecToHex(-2, 2) produces "FFFE"
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
string Function DecToHex(int dec, int numBytes) global native


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Converts a hexadecimal string into its integer representation.
;; This function performs the reciprocal action of DecToHex():
;;
;;  HexToDec("0A")          produces 10
;;  HexToDec("0000000A")    produces 10
;;  HexToDec("FFFE")        produces -2
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
int Function HexToDec(string hex) global native
