#include "skse/PapyrusNativeFunctions.h"
#include "common/ICriticalSection.h"
#include "skse/SafeWrite.h"
#include "skse/GameData.h"
#include <filesystem>
#include <fstream>

namespace lznsAVFix {
	/**
	 * Provides a sneaky way to extend Actor's functionality without recompiling SKSE itself.
	 * The GetActorValueModifier() function **IS** a member of Actor (as determined by reverse engineering),
	 *  but the SKSE team never identified or included this function as of the latest release.
	 * Ideally, the the function should be appended to Actor within skse/GameReferences.h like so:
	 *
	 *     DEFINE_MEMBER_FN(GetActorValueModifier, float, 0x6DE980, UInt32 modType, UInt32 actorValue);
	 *
	 * However, the above would require SKSE to be recompiled, which is not ideal.
	 * Creating a subclass "ActorExtended" and dynamically casting from Actor* to ActorExtended*
	 *  does NOT work because such a cast produces undefined behavior per the C++ specification.
	 * In most cases, attempting the latter will cause a crash.
	 *
	 * The workaround is to first extend Actor as shown below.
	 * All subsequent functions must then assume the extended type, instead of the original type;
	 *  no casting needs to be performed.
	 * This behavior has been verified working both on engine functions and when passing actor instances
	 *  within Papyrus scripts (again, no casting is needed).
	 *
	 * The reason this works is that SKSE never directly manipulates the Actor type; not even in Papyrus.
	 * Instead, raw pointers from memory (see assembly below) are passed as parameters to all SKSE functions,
	 *  and are simply assumed (as determined by reverse engineering) to be Actors.
	 * As long as the extended member function is **truly** a member of Skyrim's underlying actor representation,
	 *  it is correct to assume these raw pointers are the extended Actor class, rather than SKSE's implementation.
	 * In other words, all skse/Actor instances ARE actually of the extended type below.
	 * Direct casts do not work because in essence, you would be asking C++ to cast an object from itself to itself
	 *  (implying a type is actually two different types, which is impossible - hence the undefined behavior).
	 */
	class ActorExtended : public ::Actor {
	public:
		MEMBER_FN_PREFIX(ActorExtended);
		DEFINE_MEMBER_FN(GetActorValueModifier, float, 0x6DE980, UInt32 modType, UInt32 actorValue);
	};

	/**
	 * Obtains the actor value percent by incorporating the temporary modifier (normally omitted by the game).
	 * The offsets defined above account for all Condition Functions in the Creation Kit, and the associated item for Papyrus.
	 */
	float GetAVPercent(ActorExtended* actor, UInt32 avIndex);

	/**
	 * Obtains the actor value's maximum value by incorporating the temporary modifier (normally omitted by the game).
	 * May return a negative value or zero.
	 * Always returns -1 if actor is not defined.
	 */
	float GetAVMaximum(ActorExtended* actor, UInt32 avIndex);

	/** Obtains the actor's temporary AV modifier. */
	float GetAVModifier(StaticFunctionTag* base, ActorExtended* actor, UInt32 modType, BSFixedString actorValue);

	/** Installs the core mod. */
	void Install();

	/** Registers all new Papyrus functions into the plugin. */
	bool RegisterFuncs(VMClassRegistry* registry);
}
