#include "LZHealthGating.h"

#include "semaphore.h"
#include <unordered_map>

namespace lznsHealthGating {
	static ProjectIni* ini = 0;
	static Semaphore lock;
	static std::unordered_map<uintptr_t, long long> map;

	static bool runCleaner;
	static std::chrono::duration SLEEP_MILLIS = std::chrono::milliseconds(10000); //overridden by INI

	static const UInt32 AV_HEALTH = 0x18;
	static const UInt32 AV_MAGICKA = 0x19;
	static const UInt32 AV_STAMINA = 0x1A;

	static const UInt32 HOOK_DAMAGE = 0x6AC333;
	static const UInt32 HOOK_DAMAGE_RETURN = 0x6AC33A;

	/** A thread for periodically cleaning the players for which a health gate event has occurred and is providing temporary immunity. */
	static void mapCleaner() {
		std::vector<uintptr_t> toRemove;

		while (runCleaner) {
			std::this_thread::sleep_for(SLEEP_MILLIS);
			lock.acquire();

			auto millis = std::chrono::duration_cast<std::chrono::milliseconds>(std::chrono::system_clock::now().time_since_epoch()).count();

			auto it = map.begin();
			while (it != map.end()) {
				if (millis - it->second >= ini->immunityTimeMillis) {
					toRemove.push_back(it->first);
				}
				it++;
			}

			for (auto i : toRemove) {
				map.erase(i);
			}
			toRemove.clear();

			lock.release();
		}
	}

	/**
	 * Obtains the health-gated amount of damage to be dealt.
	 * The result will always be positive and never less than zero.
	 */
	static float getHealthGate(lznsAVFix::ActorExtended* target) {
		const float curr = target->actorValueOwner.GetCurrent(AV_HEALTH);
		const float max = lznsAVFix::GetAVMaximum(target, AV_HEALTH);
		const float desired = max * ini->resultHealthPercent;

		return (std::max)(curr - desired, 0.0f);
	}

	/** Determines whether the requested (rightmost) bit has been set. */
	static bool isBitSet(UInt32 value, int position) {
		return (value >> position) & 1;
	}

	/** Determines whether health gating can be applied. Parameter effect is optional (may be null for physical attacks). */
	static bool canHealthGate(lznsAVFix::ActorExtended* target, float damageThisFrame, lznsAVFix::ActorExtended* attacker, ActiveEffect* effect) {
		if (damageThisFrame >= 0.0f || !target || attacker == target || (effect && !ini->enableMagic) || (!effect && !ini->enablePhysical)) {
			return false; //cannot health-gate oneself
		}

		const UInt32 mainID = (*g_thePlayer)->baseForm->formID;
		if (ini->forMainPlayerOnly && mainID != target->baseForm->formID || (attacker && mainID == attacker->baseForm->formID)) {
			return false; //main player's attacks cannot be gated (would prevent sneak attacks, some poisons, etc.)
		}

		const float damage = std::fabs((effect) ? effect->magnitude : damageThisFrame);
		const float currHealth = target->actorValueOwner.GetCurrent(AV_HEALTH);
		if (damage < currHealth) {
			return false; //make sure the attack/effect actually does enough damage to require gating
		}

		const float currHealthPercent = lznsAVFix::GetAVPercent(target, AV_HEALTH);
		if (currHealthPercent < ini->minHealthPercent) {
			return false; //must have enough health to be gated
		}

		if (attacker) {
			if (ini->requireBelowAttackerHealth) {
				const float curr = lznsAVFix::GetAVPercent(attacker, AV_HEALTH);
				if (curr >= ini->attackerHealthPercent) {
					return false; //verify whether the attacker has low-enough health to trigger gating
				}
			}
			if (ini->requireBelowAttackerStamina) {
				const float curr = lznsAVFix::GetAVPercent(attacker, (effect && ini->swapAttackerStaminaMagicka) ? AV_MAGICKA : AV_STAMINA);
				if (curr >= ini->attackerStaminaPercent) {
					return false; //verify whether the attacker has low-enough stamina or magicka to trigger gating 
				}
			}
		}

		return true;
	}

	/**
	 * Applies health gating to physical attacks (melee and ranged).
	 * The damage argument represents the actual damage to be applied (after perks, armor, etc.).
	 * Weapon enchantments are applied separately as a magic effect, to be handled via healthGateMagic().
	 */
	static float healthGatePhysical(lznsAVFix::ActorExtended* target, float damage, lznsAVFix::ActorExtended* attacker, 
			bool bypassAll, const uintptr_t targetPointer, const long long millis) {
		if (!bypassAll && !canHealthGate(target, damage, attacker, 0)) {
			return damage;
		}
		
		if (!bypassAll) {
			map[targetPointer] = millis;
		}
		const float result = -getHealthGate(target); //must be negative
		return (bypassAll) ? 0.0f : result;
	}

	/**
	 * Applies health gating to a magic effect that was just applied to an actor.
	 * This function captures hostile magic as well as injested ingredients and weapon enchantments.
	 */
	static float healthGateMagic(lznsAVFix::ActorExtended* target, float damageThisFrame, lznsAVFix::ActorExtended* attacker, ActiveEffect* effect, 
			bool bypassAll, const uintptr_t targetPointer, const long long millis) {
		if (!bypassAll && !canHealthGate(target, damageThisFrame, attacker, effect)) {
			return damageThisFrame;
		}

		if (!bypassAll) {
			map[targetPointer] = millis;
		}
		const float result = (bypassAll) ? 0.0f : -getHealthGate(target); //must be negative

		if (effect->magnitude != damageThisFrame) {
			const float oldMagnitude = effect->magnitude;
			effect->magnitude = result; //affects future damage tick calculations
			return result / (oldMagnitude / damageThisFrame); //compatibility with framerate unlockers: use game's own frame pacing for damage ticks
		}

		effect->magnitude = result;
		return result;
	}

	/** Performs health gating on damageThisFrame and adjusts the total magic effect damage if applicable. */
	static float damageHealth(lznsAVFix::ActorExtended* target, float damageThisFrame, lznsAVFix::ActorExtended* attacker, void* effectOrActor) {
		lock.acquire();

		const long long millis = std::chrono::duration_cast<std::chrono::milliseconds>(std::chrono::system_clock::now().time_since_epoch()).count();
		const uintptr_t targetPointer = (uintptr_t) target;
		bool bypassAll = false;
		try {
			if (millis <= map.at(targetPointer) + ini->immunityTimeMillis) {
				bypassAll = true;
			}
		} catch (std::out_of_range ex) {}

		float result;
		if (effectOrActor == attacker || effectOrActor == target) {
			result = healthGatePhysical(target, damageThisFrame, attacker, bypassAll, targetPointer, millis);
		} else {
			result = healthGateMagic(target, damageThisFrame, attacker, (ActiveEffect*) effectOrActor, bypassAll, targetPointer, millis);
		}

		lock.release();
		return result;
	}

	/** Alters the damage dealt to an actor. */
	static void __declspec(naked) hookDamageHealth() {
		__asm {
			push eax
			push ebx
			push ecx
			push edx
			push esi
			push edi
			push ebp
			mov ebp, esp
			add ebp,0x28
			push ebp
			mov ebp,[esp+0x04]
			push [esp+0x38]		//originally ESP+0x14
			push [esp+0x34]		//originally ESP+0x0C
			push [esp+0x34]		//originally ESI+0x08
			push esi			//originally ESI
			call damageHealth
			fstp dword ptr ds:[esp+0x4]
			add esp, 0x4
			pop eax
			add esp, 0x8
			pop ebx
			mov [ebx], eax
			pop ebp
			pop edi
			pop esi
			pop edx
			pop ecx
			pop ebx
			pop eax
			add esp, 0x04
			mov ecx,esi					//first original hooked instruction
			push 0x6e0760
			add esp, 0x04
			call [esp-0x04]				//second original hooked instruction (hook function call spans two instructions in size)
			jmp HOOK_DAMAGE_RETURN
		}
	}

	/** Installs the core mod. */
	void Install(ProjectIni* iniFile) {
		ini = iniFile;

		WriteRelCall(HOOK_DAMAGE, (UInt32) hookDamageHealth);


		runCleaner = true;
		if (ini->cleanupPollingTimeMillis > 0.0f) {
			SLEEP_MILLIS = std::chrono::milliseconds(ini->cleanupPollingTimeMillis);
			std::thread thread(mapCleaner);
			thread.detach();
		}
	}

	/** Registers all new Papyrus functions into the plugin. */
	bool RegisterFuncs(VMClassRegistry* registry) {
		return true;
	}
}
