/**
 * An abstract class representing an initialization file for SKSE.
 * All such files must be located in the <Skyrim>/Data/SKSE/Plugins folder.
 * After creating the instance, call load() to populate values from disk.
 * 
 * Initialization files take the format:
 * 
 *     [Header]
 *     key1=value
 *     key2=anotherValue
 *     
 *     ; a comment
 *     [AnotherHeader]
 *     ...
 * 
 * Comments are permitted if the line is preceded with a semicolon ";".
 * Trailing comments are NOT allowed on key/value lines.
 * Whitespace around sections, keys, and values are ignored.
 */
class IniFile {
public:
	/**
	 * Loads the ini file from disk, return false if loading failed or the file does not exist.
	 * Can only be called once.
	 * Calls setDefaults() before the file is actually loaded.
	 */
	bool load();

protected:
	/** Obtains the name of the ini file, as used by load(). Must include extension. */
	virtual std::string getFilename() = 0;

	/** Assigns default values to all settings. Called on load(), only if loading has not already been done. */
	virtual void setDefaults() = 0;

	/**
	 * Puts the setting as read from the given header.
	 * Called by load() for each key/value pair.
	 * All provided values will be lower-case except for value itself.
	 */
	virtual void putSetting(std::string section, std::string key, std::string value) = 0;

	/** Called immediately at the end of successful loading. Typically used for post-procesing on values. */
	virtual void finishLoading() = 0;

	/** Converts the requested value into an integer. Invalid arguments produce 0. */
	UInt32 toInt(std::string value);

	/** Converts the requested value into a float. Invalid arguments produce 0.0. */
	float toFloat(std::string value);

	/**
	 * Converts the requested value into a boolean.
	 * The string "true" and any nonzero integer value are each considered true; everything else is false.
	 * The value will be converted to lower-case.
	 */
	bool toBool(std::string value);

private:
	std::string filePath;

	/** Creates the path for the initialization file. */
	std::string generatePath();

	/** Converts the provided string to all lower-case. */
	void toLower(std::string& str);
};
