#include "iniBase.h"

/** Represents the file containing all user-configured values. */
class ProjectIni : public IniFile {
public:
	/** Whether health gating is permitted at all. */
	bool isEnabled;

	/**
	 * Whether health gating affects only the main player.
	 * Disabling this setting is not recommended as some unexpected effects may occur e.g. in scripted scenes.
	 * Regardless of this setting, the main player's attacks are NEVER gated,
	 *  as doing so would prevent most sneak attacks from working properly.
	 */
	bool forMainPlayerOnly;

	/** Whether health gating should be applied to physical attacks (melee, ranged); NOT enchantments. */
	bool enablePhysical;

	/** Whether health gating should be applied to magic effects (enchantments and spells). */
	bool enableMagic;

	/** Whether health gating stops occurring if the attacker has high enough health. */
	bool requireBelowAttackerHealth;

	/** Same as requireBelowAttackerHealth, but checks against the attacker's stamina percentage. */
	bool requireBelowAttackerStamina;

	/**
	 * Only when the attacker is casting a spell, whether the requireBelowAttackerStamina requirement (if present)
	 *  should be substituted with a magicka requirement.
	 */
	bool swapAttackerStaminaMagicka;

	/**
	 * The player's health percentage must be greater than or equal to this amount for health gating to occur.
	 * Must be less than 1.0 and greater than resultHealthPercent or gating will be disabled.
	 */
	float minHealthPercent;

	/**
	 * The player's health will be set to this percentage if a health gating event occurs.
	 * Must be lower than minHealthPercent and greater than zero or gating will be disabled.
	 */
	float resultHealthPercent;

	/**
	 * The attacker's health percentage must be less than this amount for health gating to occur.
	 * Health and stamina are evaluated separately; both checks (if enabled) must be passed for gating to occur.
	 * If greater than 1.0 or less than or equal to zero, the health requirement will be disabled.
	 */
	float attackerHealthPercent;

	/**
	 * The attacker's stamina percentage must be less than this amount for health gating to occur.
	 * Health and stamina are evaluated separately; both checks (if enabled) must be passed for gating to occur.
	 * If greater than 1.0 or less than or equal to zero, the stamina requirement will be disabled.
	 * 
	 * If swapAttackerStaminaMagicka is set to true and the attacker is casting a spell,
	 *  the attacker's magicka percentage will be checked instead of stamina.
	 * Note that this applies to the damage dealt by enchantments on weapons as well.
	 * The physical damage component for enchanted weapon strikes always checks the stamina percentage.
	 */
	float attackerStaminaPercent;

	/** 
	 * How long a gated player should remain immune to future attacks, in milliseconds.
	 * If set to zero, weapons with enchantments and spells with multiple effects may not be fully gated.
	 * Must be larger than or equal to zero.
	 */
	UInt32 immunityTimeMillis;

	/**
	 * How frequently the cleaning thread should run, to compact the list of (no longer used) immune players.
	 * Must be larger than or equal to zero.
	 */
	UInt32 cleanupPollingTimeMillis;

protected:
	std::string getFilename() override;
	void setDefaults() override;
	void putSetting(std::string section, std::string key, std::string value) override;
	void finishLoading() override;
};
